import click
from data.main import connect, paths
import pandas as pd
from lxml import etree
from pathlib import Path
import os
import csv


@click.command(name="bias:normalize")
def normalize() -> None:
    with connect() as db:
        db.sql("""
            CREATE OR REPLACE TABLE publisher_bias AS
            WITH cte AS (
                SELECT
                    p.id as publisher_id
                    ,b.id as bias_id
                    ,b.bias as label
                    ,JARO_WINKLER_SIMILARITY(LOWER(p.name), LOWER(b.publisher)) as similarity
                FROM bias_ratings b
                JOIN top.publishers p
                ON JARO_WINKLER_SIMILARITY(LOWER(p.name), LOWER(b.publisher)) > 0.95
            ),ranked AS (
                SELECT
                    publisher_id
                    ,bias_id
                    ,label
                    ,similarity
                    ,ROW_NUMBER() OVER(PARTITION BY publisher_id ORDER BY similarity DESC) AS rn
                FROM cte
            )
            SELECT
                publisher_id
                ,label
                ,bias_id
            FROM ranked
            WHERE ranked.rn = 1
        """)

    mapping = [
            {'label' :'left' , 'ordinal': -2},
            {'label' :'left-center' , 'ordinal': -1},
            {'label' :'center' , 'ordinal': 0},
            {'label' :'right-center' , 'ordinal': 1},
            {'label' :'right' , 'ordinal': 2},
    ]
    mapping = pd.DataFrame(mapping)

    with connect() as db:
        db.query("alter table bias_ratings add column ordinal int")
        db.query("""
            update bias_ratings b
            set ordinal = o.ordinal
            FROM mapping o
            WHERE o.label = b.bias
        """)


@click.command(name='bias:parse')
def parse() -> None:
    """parse the save html page of allslides.com bias ratings into a normalized csv file"""
    bias_html = paths('data') / 'allsides.html'

    parser = etree.HTMLParser()
    tree = etree.parse(str(bias_html), parser)
    root = tree.getroot()
    rows = root.xpath('//table[contains(@class,"views-table")]/tbody/tr')

    ratings = []
    for row in rows:
        rating = dict()
        publisher = row.xpath('./td[contains(@class, "source-title")]/a')[0].text
        rating['publisher'] = publisher

        bias = row.xpath('./td[contains(@class, "views-field-field-bias-image")]/a')[0].get('href')
        bias = bias.split('/')[-1]
        rating['bias'] = bias

        agree = row.xpath('.//span[contains(@class, "agree")]')[0].text
        disagree = row.xpath('.//span[contains(@class, "disagree")]')[0].text

        rating['agree'] = int(agree)
        rating['disagree'] = int(disagree)
        ratings.append(rating)
    df = pd.DataFrame(ratings)
    df.to_csv(paths('data') / 'bias_ratings.csv', sep="|", index=False, quoting=csv.QUOTE_NONNUMERIC)

@click.command(name="bias:load")
def load() -> None:
    f = str(paths('data') / "bias_ratings.csv")

    with connect() as db:
        db.sql(f"""
            CREATE TABLE bias_ratings as 
            select 
                row_number() over(order by b.publisher) as id
                ,b.*
            from read_csv_auto('{f}') b
        """)

@click.command('bias:export')
def export():
    with connect() as db:
        all_bias = db.query("""
            SELECT
                id as bias_id
                ,publisher as name
                ,bias as label
            FROM bias_ratings
            ORDER by agree desc
        """)

    all_bias.df().to_csv(paths('data') / 'TMP_publisher_bias.csv', sep="|", index=False)
    with connect() as db:
        mapped_bias = db.query("""
            SELECT
                p.id as publisher_id
                ,p.name as name
                ,p.tld as tld
                ,b.label as bias
                ,b.bias_id as bias_id
            FROM top.publishers p
            LEFT JOIN publisher_bias b
            ON b.publisher_id = p.id
    """)
    mapped_bias.df().to_csv(paths('data') / 'TMP_publisher_bias_to_load.csv', sep="|", index=False)

@click.command('bias:import-mapped')
def import_mapped():
    table_name = "top.publisher_bias"

    df = pd.read_csv(paths('data') / 'TMP_publisher_bias_to_load.csv', sep="|")

    with connect() as db:
        db.query(f"""
            CREATE OR REPLACE TABLE {table_name} AS
            SELECT
                publisher_id AS publisher_id
                ,cast(bias_id AS int) as bias_id
            FROM df
            WHERE bias_id IS NOT NULL
        """)

    print(f"created table: {table_name}")

