import click
from data.main import connect, paths
import os
import seaborn as sns
import matplotlib.pyplot as plt
from pathlib import Path
import numpy as np

@click.command('plot:articles-per-year')
def articles_per_year():
    save_to = paths('figures') / 'articles_per_year.png'

    with connect() as db:
        data = DB.query("""
            select
                year(published_at) as year
                ,count(1) as stories
            from stories
            group by 
                year(published_at)
        """).df()

    ax = sns.barplot(x=data.year, y=data.stories, color='tab:blue')
    ax.tick_params(axis='x', rotation=90)
    ax.set(title="count of articles per year", ylabel="count of stories (#)")
    plt.tight_layout()
    plt.savefig(save_to)
    print(f"saved: {save_to}")

@click.command('plot:distinct-publishers')
def distinct_publishers():
    save_to = paths('figures') / 'distinct_publishers.png'

    with connect() as db:
        data = DB.query("""
            select
                year(published_at) as year
                ,count(distinct publisher_id) as publishers
            from stories
            group by 
                year(published_at)
        """).df()

    ax = sns.barplot(x=data.year, y=data.publishers, color='tab:blue')
    ax.tick_params(axis='x', rotation=90)
    ax.set(title="count of publishers per year", ylabel="count of publishers (#)")
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")

@click.command('plot:stories-per-publisher')
def stories_per_publisher():
    save_to = paths('figures') / 'stories_per_publisher.png'

    with connect() as db:
        data = db.query("""
            with cte as (
            select
                ps.publisher_id
                ,year(s.published_at) as year
                ,count(1) as stories
            from stories s
            join mbfc.publisher_stories ps
            on ps.story_id = s.id
            group by 
                ps.publisher_id
                ,year(s.published_at)
            ) , agg as (
                select
                    publisher_id
                    ,avg(stories) as stories_per_year
                    ,case 
                        when avg(stories) < 2 then 2
                        when avg(stories) < 4 then 4
                        when avg(stories) < 8 then 8
                        when avg(stories) < 16 then 16
                        when avg(stories) < 32 then 32
                        when avg(stories) < 64 then 64
                        when avg(stories) < 128 then 128
                        else 129
                    end as max_avg
                from cte
                group by 
                    publisher_id
            )
            select
                max_avg
                ,count(1) as publishers
            from agg
            group by
                max_avg
        """).df()

    ax = sns.barplot(x=data.max_avg, y=data.publishers, color='tab:blue')
    ax.set(title="histogram of publisher stories per year", ylabel="count of publishers (#)", xlabel="avg. stories / year", xticklabels=['2', '4', '8', '16', '32', '64', '128', '>128'])
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")


@click.command('plot:top-publishers')
def top_publishers():
    """plot top publishers over time"""

    save_to = paths('figures') / 'top_publishers.png'

    with connect() as db:
        db.query("""
                SELECT
                    p.tld
                    ,p.id
                FROM mbfc.publishers p
                JOIN mbfc.publisher_stories ps
                ON ps.publisher_id = p.id
                JOIN stories s
                ON s.id = ps.story_id
                GROUP BY
                    p.tld
                    ,p.id
                order by count(1) desc
                limit 20
            """)

    with connect() as db:
        data = db.query("""
            WITH p as ( 
                SELECT
                    p.tld
                    ,p.id
                FROM mbfc.publishers p
                JOIN mbfc.publisher_stories ps
                ON ps.publisher_id = p.id
                JOIN stories s
                ON s.id = ps.story_id
                GROUP BY
                    p.tld
                    ,p.id
                order by count(1) desc
                limit 20
            ) 
            SELECT
                p.tld
                ,YEAR(s.published_at) AS year
                ,COUNT(1) AS stories
            FROM stories s
            JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            JOIN p
            ON p.id = ps.publisher_id
            GROUP by 
                p.tld
                ,YEAR(published_at)
            ORDER BY year, COUNT(DISTINCT s.id) DESC
        """).df()
    
    pivot = data.pivot(columns='year', index='tld', values='stories')
    ax = sns.heatmap(pivot, cmap="crest")
    ax.set(title="top 20 publishers (by tld)", ylabel="tld", xlabel="stories / year (#)")
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")


@click.command('plot:common_tld')
def common_tld():
    import dataframe_image as dfi
    save_to = paths('figures') / 'common_tld.png'

    with connect() as db:
        data = db.query("""
            select
                split_part(url, '.', -1) as tld
                ,count(1) as publishers
                ,case when count(1) < 20
                    then string_agg(distinct url, '\t')
                    else NULL
                    end as urls
            from publishers
            group by
                split_part(url, '.', -1)
            order by 
                count(1) desc
        """).df()
    data[:15][['tld', 'publishers']].style.hide(axis="index").export_png(save_to, table_conversion='matplotlib')

def stats():

    # raw
    DB.query("""
        SELECT
            'total stories' as key
            ,COUNT(1) as value
        FROM stories
        UNION
        SELECT
            'total related' as key
            ,COUNT(1) as value
        FROM related_stories
        UNION
        SELECT
            'top level domains' as key
            ,COUNT(distinct tld) as value
        FROM stories
        UNION
        SELECT
            'publishers' as key
            ,COUNT(1) as value
        FROM publishers
        UNION
        SELECT
            'authors' as key
            ,COUNT(distinct author) as value
        FROM stories
        UNION
        SELECT
            'min year' as key
            ,min(year(published_at)) as value
        FROM stories
        UNION
        SELECT
            'max year' as key
            ,max(year(published_at)) as value
        FROM stories
    """).df().to_markdown(index=False)

    # selected
    DB.query("""
        SELECT
            'total stories' as key
            ,COUNT(1) as value
        FROM top.stories
        UNION
        SELECT
            'total related' as key
            ,COUNT(1) as value
        FROM top.related_stories
        UNION
        SELECT
            'top level domains' as key
            ,COUNT(distinct tld) as value
        FROM top.stories
        UNION
        SELECT
            'publishers' as key
            ,COUNT(1) as value
        FROM top.publishers
        UNION
        SELECT
            'authors' as key
            ,COUNT(distinct author) as value
        FROM top.stories
        UNION
        SELECT
            'min year' as key
            ,min(year(published_at)) as value
        FROM top.stories
        UNION
        SELECT
            'max year' as key
            ,max(year(published_at)) as value
        FROM top.stories
    """).df().to_markdown(index=False)

@click.command('plot:bias-stats')
def bias_stats():
    import dataframe_image as dfi
    save_to = paths('figures') / 'bias_stats.png'

    DB = connect()

    df = DB.query("""
        SELECT
            string_agg(distinct bias)
        FROM bias_ratings
    """).df()

    df.keys()

    df = DB.query("""
        SELECT
            'publishers' as metric
            ,count(1) as value
        FROM bias_ratings
        UNION
        SELECT
            'labels' as metric
            ,count(distinct bias) as value
        FROM bias_ratings
        UNION
        SELECT
            'right' as metric
            ,count(1) as value
        FROM bias_ratings
        WHERE bias in ('right', 'right-center')
        UNION
        SELECT
            'left' as metric
            ,count(1) as value
        FROM bias_ratings
        WHERE bias in ('left', 'left-center')
        UNION
        SELECT
            'center' as metric
            ,count(1) as value
        FROM bias_ratings
        WHERE bias in ('center')
        UNION
        SELECT
            'agree_range' as metric
            ,'[' 
                || min(cast(agree as float) / (agree + disagree)) 
                || '-' 
                || max(cast(agree as float) / (agree + disagree)) 
                || ']'
                as value
        FROM bias_ratings
        WHERE bias in ('center')
    """).df()
    DB.close()
    print(df.to_markdown(index=False))

@click.command('plot:bias-over-time')
def bias_over_time():
    """plot bias labels over time"""

    save_to = paths('figures') / 'bias_over_time.png'

    with connect() as db:
        df = db.sql("""
            SELECT
                p.bias
                ,p.id
                ,date_trunc('year', s.published_at) as year
                ,count(1) as stories
            FROM stories s
            JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            JOIN mbfc.publishers p
            ON p.id = ps.publisher_id
            where year(s.published_at) not in (2006, 2023)
            and p.ordinal != -1
            GROUP BY
                p.bias
                ,p.id
                ,p.ordinal
                ,date_trunc('year', s.published_at)
            order by 
                p.ordinal
                ,date_trunc('year', s.published_at)
        """).df()

    ax = sns.relplot(df, kind='line', x='year', y='stories', col='bias', units='id', estimator=None, palette='rainbow')
    ax.set(ylabel="stories", xlabel="year")
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")

def bias_missing():

    with connect() as db:
        df = db.sql("""
            SELECT
                date_trunc('year', s.published_at) as year
                ,s.tld
                ,count(1) as stories
            FROM stories s
            LEFT JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            WHERE ps.publisher_id is NULL
            AND year(s.published_at) not in (2006, 2023)
            GROUP BY
                s.tld
                ,date_trunc('year', s.published_at)
            HAVING count(1) > 10
            ORDER BY 
                date_trunc('year', s.published_at)
        """).df()

        ax = sns.lineplot(df, x='year', y='stories', units='tld', estimator=None)
        ax.set(ylabel="stories", xlabel="year")
        plt.tight_layout()
        plt.show()
        #plt.savefig(save_to)
        plt.close()
        #print(f"saved: {save_to}")
