import click
from data.main import connect, paths, ticklabels
import os
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

@click.command('plot:emotion-over-time')
def emotion_over_time():

    filename = "emotion_over_time.png"
    save_to = paths('figures') / filename

    with connect() as db:
        emotions = db.sql("""
            SELECT
                date_trunc('year', s.published_at) AS year
                ,e.label AS emotion
                ,count(1) AS stories
            FROM stories s
            JOIN story_emotions se
            ON s.id = se.story_id
            JOIN emotions e
            ON e.id = se.emotion_id
            GROUP by
                date_trunc('year', s.published_at)
                ,e.label
        """).df()

    ax = sns.scatterplot(x=emotions['year'], y=emotions['stories'], hue=emotions['emotion'])
    ax.set(title="title emotions over years", xlabel="year", ylabel="stories (#)")
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")
    os.system(f'xdg-open {save_to}')

@click.command('plot:emotion-regression')
def emotion_regression():
    """plot emotion over time as regression"""

    from sklearn import linear_model
    from sklearn.model_selection import train_test_split
    from sklearn.metrics import ConfusionMatrixDisplay

    filename = "emotion_regression.png"
    save_to = paths('figures') / filename


    with connect() as db:
        #emotions = db.query("""
        #    SELECT
        #        label
        #    FROM emotions e
        #""").df()['label'].to_list()
        df = db.sql(f"""
            SELECT
                epoch(date_trunc('yearweek', s.published_at)) AS date
                ,e.id AS emotion_id
                ,p.id as publisher_id
                ,count(1) AS stories
            FROM stories s
            JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            JOIN mbfc.publishers p
            ON p.id = ps.publisher_id
            JOIN story_emotions se
            ON s.id = se.story_id
            JOIN emotions e
            ON e.id = se.emotion_id
            WHERE p.ordinal != -1
            GROUP by
                epoch(date_trunc('yearweek', s.published_at))
                ,p.id
                ,e.id
        """).df()

    results = []
    for (emotion_id, publisher_id), group in df.groupby(['emotion_id', 'publisher_id']):
        model = linear_model.LinearRegression()
        x = group['date'].to_numpy().reshape(-1, 1)
        y = group['stories'].to_numpy()
        model.fit(x, y)
        per_year = model.coef_.item() * 60 * 60 * 24 * 365
        results.append({'emotion_id' : emotion_id, 'publisher_id':publisher_id, 'per_year' : per_year})
    results = pd.DataFrame(results)

    with connect() as db:
        out = db.query("""
            SELECT
                e.label as emotion
                ,avg(results.per_year) as avg_reg_coef
                ,p.bias
            FROM results
            JOIN emotions e
            ON e.id = results.emotion_id
            JOIN mbfc.publishers p
            ON p.id = results.publisher_id
            GROUP BY
                e.label
                ,p.bias
        """).df()

    pivot = out.pivot(index=['emotion'], columns=['bias'], values=['avg_reg_coef'])

    ax = sns.heatmap(pivot, cmap='BrBG', vmin=-0.01, vmax=0.01, center=0)
    #ax = sns.heatmap(pivot, cmap='RdBu_r', center=0)
    ax.set(title="slope of regression (stories/year) by bias and emotion"
           ,xticklabels=ticklabels()
           ,xlabel="bias"
           ,ylabel="emotion")
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")

@click.command('plot:emotion-hist')
def emotion_hist():

    filename = "emotion_hist.png"
    save_to = paths('figures') / filename

    with connect() as db:
        data = db.sql("""
            SELECT
                p.bias
                ,count(1) as stories
            FROM stories s
            JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            JOIN mbfc.publishers p
            ON p.id = ps.publisher_id
            WHERE p.ordinal != -1
            GROUP BY
                p.bias
        """).df()

    ax = sns.barplot(data, x='bias', y='stories', palette='rainbow', order=ticklabels())
    ax.set(title="count of stories per bias rating", xlabel="bias rating", xticklabels=ticklabels())
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")
