import click
from data.main import connect, paths, ticklabels
import seaborn as sns
import matplotlib.pyplot as plt

@click.command('plot:sentiment-over-time')
def over_time():

    filename = "sentiment_over_time.png"
    save_to = paths('figures') / filename

    with connect() as db:
        data = db.sql("""
            SELECT
                avg(sent.class_id) as sentiment
                ,s.published_at as date
            FROM top.story_sentiments sent
            JOIN top.stories s
            ON s.id = sent.story_id
            GROUP BY
                s.published_at
        """).df()

    ax = sns.scatterplot(x=data['date'], y=data['sentiment'])
    ax.set(title="sentiment vs. time")
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")

@click.command('plot:bias-vs-sentiment-over-time')
def bias_over_time():
    """plot sentiment/bias vs. time"""

    filename = "bias_vs_sentiment_over_time.png"
    save_to = paths('figures') / filename

    with connect() as db:
        data = db.sql("""
            with cte as (
                SELECT
                    avg(sent.class_id) as sentiment
                    ,date_trunc('yearweek', s.published_at) as date
                    ,p.bias
                FROM story_sentiments sent
                JOIN stories s
                ON s.id = sent.story_id
                JOIN mbfc.publisher_stories ps
                ON ps.story_id = s.id
                JOIN mbfc.publishers p
                ON p.id = ps.publisher_id
                WHERE p.ordinal != -1
                GROUP BY
                    date_trunc('yearweek', s.published_at)
                    ,p.bias
            )
            SELECT
                median(sentiment) OVER (PARTITION BY bias ORDER BY date DESC ROWS BETWEEN 0 PRECEDING AND 7 FOLLOWING) as sentiment
                ,date
                ,bias
            FROM cte
            WHERE year(date) not in (2005, 2023)
        """).df()

    #ax = sns.relplot(data, x='date', y='sentiment', col='bias', palette='rainbow', hue='bias', col_order=ticklabels())
    ax = sns.lineplot(data, x='date', y='sentiment', palette='rainbow', hue='bias', hue_order=ticklabels())
    plt.axhline(y=0.5, color='black', linestyle='--', label='neutral') 
    ax.set(title='sentiment and bias vs. time', ylabel='8 week rolling avg. sentiment', xlabel='date')
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")

@click.command('plot:sentiment-recent-winner')
def bias_vs_recent_winner():
    """plot bias vs. distance to election"""

    filename = "bias_vs_recent_winner.png"
    save_to = paths('figures') / filename

    with connect() as db:
        data = db.sql("""
            SELECT
                round(e.days_away, -1) as days_away
                ,p.bias
                ,avg(sent.class_id) as sentiment
                ,count(1) as stories
            FROM stories s
            JOIN story_sentiments sent
            ON s.id = sent.story_id
            JOIN election_distance e
            ON e.publish_date = s.published_at
            JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            JOIN mbfc.publishers p
            ON p.id = ps.publisher_id
            GROUP BY
                round(e.days_away, -1)
                ,p.bias
        """).df()

    ax = sns.scatterplot(data, x='days_away', y='sentiment', hue='bias', hue_order=ticklabels(), palette='rainbow')
    ax.set(title="sentiment vs. days to nearest election", xlabel="days to nearest election", ylabel="average title seniment")
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")

@click.command('plot:sentiment-hist')
def sentiment_hist():

    filename = "sentiment_hist.png"
    save_to = paths('figures') / filename

    with connect() as db:
        data = db.sql("""
            SELECT
                p.bias
                ,count(1) as stories
            FROM stories s
            JOIN mbfc.publisher_stories ps
            ON ps.story_id = s.id
            JOIN mbfc.publishers p
            ON p.id = ps.publisher_id
            WHERE p.ordinal != -1
            GROUP BY
                p.bias
        """).df()

    ax = sns.barplot(data, x='bias', y='stories', hue='bias', palette='rainbow')
    ax.set(title="count of stories per bias rating", xlabel="bias rating", xticklabels=ticklabels())
    plt.tight_layout()
    plt.savefig(save_to)
    plt.close()
    print(f"saved: {save_to}")
