import click
from tqdm import tqdm
from enum import Enum, auto
from dotenv import load_dotenv
import os

import torch
from torch import nn
from torch import optim
from torch.utils.data import DataLoader
from accelerate import Accelerator

from train.dataset import NewsDataset
from train.model import Classifier
#from model.linear import LinearClassifier

class Stage(Enum):
    TRAIN = auto()
    DEV = auto()

@click.command('train:main')
def main():
    dev_after = 20
    visible_devices = None
    lr = 1e-4
    epochs = 10
    debug = False
    torch.manual_seed(0)
    num_workers = 0

    embedding_length = int(os.getenv('EMBEDDING_LENGTH', 384))

    dataset = NewsDataset()
    trainset, devset = torch.utils.data.random_split(dataset, [0.8, 0.2])
    batch_size = 512
    trainloader = DataLoader(trainset, batch_size=batch_size, shuffle=True, num_workers=num_workers, drop_last=True)
    devloader = DataLoader(devset, shuffle=False, num_workers=num_workers)
    accelerator = Accelerator()
    model = Classifier(embedding_length=embedding_length, classes=5)

    # it's possible to control which GPUs the process can see using an environmental variable
    if visible_devices:
        os.environ['CUDA_VISIBLE_DEVICES'] = visible_devices
    if debug:
        os.environ['CUDA_LAUNCH_BLOCKING'] = '1'
        #accelerator.log({"message" :"debug enabled"})

    criterion = torch.nn.CrossEntropyLoss()
    optimizer = torch.optim.Adam(model.parameters(), lr=lr)

    # wrap objects with accelerate
    model, optimizer, trainloader, devloader = accelerator.prepare(model, optimizer, trainloader, devloader)
    def run():
        """runner for training and valdating"""
        running_loss = 0.0
        # set the model to train model
        model.train() if stage == Stage.TRAIN else model.eval()
        dataloader = trainloader if stage == Stage.TRAIN else devloader
        desc = 'train epoch' if stage == Stage.TRAIN else 'dev epoch'
        if debug:
            ...

        # Make sure there are no leftover gradients before starting training an epoch
        optimizer.zero_grad()

        for batch, (x, y) in enumerate(tqdm(dataloader, desc=desc)):
            pred_y = model(x)  # Forward pass through model
            loss = criterion(pred_y, y)
            running_loss += loss                        # Increment running loss
            # Only update model weights on training
            if stage == Stage.TRAIN:
                accelerator.backward(loss)             # Increment gradients within model by sending loss backwards
                optimizer.step()                       # Update model weights
                optimizer.zero_grad()                  # Reset gradients to 0
        return running_loss / len(dataloader)


    for epoch in range(epochs):
        if (epoch - 1) % dev_after == 0:
            stage = Stage.DEV
            log = run()
            print(f"dev loss: {log}")
        else:
            stage = Stage.TRAIN
            log = run()
            print(f"train loss: {log}")

    breakpoint()
    from data.main import data_dir, connect
    import numpy as np
    import pandas as pd
    from bias import int_to_label

    embeddings = dataset.embeddings
    embedding_ids = dataset.data

    DB = connect()
    query = """
        SELECT
            s.id
            ,title
            ,p.name
            ,count(1) over (partition by publisher_id) as stories
        FROM stories s
        JOIN publishers p
        on p.id = s.publisher_id
        WHERE s.publisher_id NOT IN (
            SELECT
                id 
            FROM publisher_bias b
        )
    """
    data = DB.sql(query).df()
    embeddings = np.load(data_dir() / 'embeddings.npy')
    embedding_ids = pd.DataFrame(np.load(data_dir() / 'embedding_ids.npy'), columns=['id']).reset_index()


    for i in range(10):
        embedding =  embeddings[embedding_ids[embedding_ids['id'] == data.iloc[i]['id']]['index']]
        title = data.iloc[i]['title']
        publisher = data.iloc[i]['name']
        class_pred = nn.functional.softmax( model(torch.tensor(embedding))).detach()
        class_id = int(torch.argmax(nn.functional.softmax( model(torch.tensor(embedding))).detach()))
        print(f"{publisher}: {int_to_label(class_id)} - \"{title}\"")

    embedding_ids['id'] == data.iloc[0]['id']
    embedding_ids[embedding_ids['id'] == data.iloc[0]['id']]
    embedding =  embeddings[embedding_ids[embedding_ids['id'] == data.iloc[0]['id']]['index']]
    title
    publisher

    model().get_last_layer(torch.tensor(embedding))
